package ispn;

import java.util.Map;
import java.util.Set;

import javax.annotation.PreDestroy;

import org.apache.log4j.Logger;
import org.infinispan.AdvancedCache;
import org.infinispan.config.ConfigurationValidatingVisitor;
import org.infinispan.config.InfinispanConfiguration;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.manager.EmbeddedCacheManager;
import org.infinispan.util.FileLookup;
import org.jboss.ejb3.annotation.Service;

@Service
public class CacheManagerHolder {

	private Logger logger = Logger.getLogger("CacheContainer");
	private final String INFINISPAN_CONFIGURATION = "my-infinispan.xml";
	private static EmbeddedCacheManager cacheManager = null;
	private static final String CACHE_NAME = "stateretrieval-file";
	
	public CacheManagerHolder() {
		logger.info("***** CacheContainer created ****");
		diagnostic();
	}
	
	
    /*
     * Automatically called by JBoss on startup
     */
    public void start() throws Exception {
    	logger.info("***** CacheContainer started ****");

		diagnostic();
		
		logger.info("** Initialize Infinispan Cache from: "+INFINISPAN_CONFIGURATION);
		cacheManager = new DefaultCacheManager(INFINISPAN_CONFIGURATION);
		
		logger.info("** Starting Infinispan CacheManager");
		cacheManager.start();
		
		logger.info("** Starting Infinispan Default Cache");
		getCache().start();
		
		logger.info("** ****************************************");
    }

    /*
     * Automatically called by JBoss before shutdown or undeployment
     */
    public void stop() {
    	logger.info("***** CacheContainer stopped ****");
    }
    

	@PreDestroy
	public void destroy() {
		logger.info("** Closing Infinispan CacheManager");
		cacheManager.stop();
	}
	
	private void diagnostic() {
		try {
			logger.info("** Infinispan Version: "+org.infinispan.Version.printVersion());
			logger.info("** Infinispan SchemaSystemProperty: "+System.getProperty(InfinispanConfiguration.SCHEMA_SYSTEM_PROPERTY));
			
			
			String schemaFilename = InfinispanConfiguration.resolveSchemaPath();
			logger.info("** Infinispan - Schema FileName="+schemaFilename);
			
			FileLookup lookup = new FileLookup();
			logger.info("** Infinispan - Schema URL="+lookup.lookupFileLocation(schemaFilename));
			logger.info("** Infinispan - ConfigurationFile URL="+lookup.lookupFileLocation(INFINISPAN_CONFIGURATION));
			
	        InfinispanConfiguration configuration = InfinispanConfiguration.newInfinispanConfiguration(
	        		INFINISPAN_CONFIGURATION, InfinispanConfiguration.resolveSchemaPath(),
	                new ConfigurationValidatingVisitor());
		}
		catch(Exception e) {
			//
		}
	}
	
	public EmbeddedCacheManager getCacheManager() {
		return cacheManager;
	}
	
	public AdvancedCache getCache() {
		return getCacheManager().getCache(CACHE_NAME).getAdvancedCache();
	}
}